/*
**	This file is part of cfgd
**
**      Copyright 2002 Oliver Schroeder
**
**      Permission to use, copy, modify, and distribute this
**      software and its documentation for any purpose and
**      without any fee is hereby granted, provided that the
**      above copyright notice and this permission notice
**      appear in all copies of the software and derivative
**      works or modified versions thereof, and that both the
**      copyright notice and this permission and disclaimer
**      notice appear in supporting documentation.
**
**      This software is provided "as is" WITHOUT ANY WARRANTY.
**
**      Oliver Schroeder <post@o-schroeder.de>
**
*/

#ifndef CCONFIG_HEADER_H
#define CCONFIG_HEADER_H

#include "cfgstack.h"

/*********************************************************************
 * these structures should not be accessible by the API user. So
 * they are declared in cconfig.c.
 *********************************************************************/
struct st_config;
typedef struct st_config t_config;

/*********************************************************************/
/**@name eINTERNALFLAGS
 * 
 *	@see i_CFG_ISLOCAL
 *	@see i_CFG_ISCONFIG
 *	@see i_CFG_ISMODIFIED
 */
/*********************************************************************/
typedef enum eCFGINTERNALFLAGS
{
	/** this cfg is local to this host			*/
	i_CFG_ISLOCAL		= 1,
	/** the caller is a configurator, not a normal client	*/
	i_CFG_ISCONFIG		= 2,
	/** the config has been modified			*/
	i_CFG_ISMODIFIED	= 4
} i_CFGINTERNALSFLAGS;

/*********************************************************************/
/**@name eCFGERRORs
 * 
 * 	possible errorcodes set by the library functions
 *	@see eCFGERROR_OK
 *	@see eCFGERROR_NO_CREATE
 *	@see eCFGERROR_NO_SECTION
 *	@see eCFGERROR_NO_VARIABLE
 *	@see eCFGERROR_OUT_OF_MEMORY
 *	@see eCFGERROR_PARSE_ERROR
 *	@see eCFGERROR_UNKNOWN
 */
/*********************************************************************/
typedef enum eCFGERRORs
{
/*@{*/
	/** everything alright			*/
	CFGERROR_OK	=  0,
	/** config does not exist		*/
	CFGERROR_NO_CREATE,
	/** section does not exist		*/
	CFGERROR_NO_SECTION,
	/** variable does not exist		*/
	CFGERROR_NO_VARIABLE,
	/** no more memory			*/
	CFGERROR_OUT_OF_MEMORY,
	/** illegal instruction in config file	*/
	CFGERROR_PARSE_ERROR,
	/** no other error fits			*/
	CFGERROR_UNKNOWN
/*@}*/
} i_CFGERROR;

/*********************************************************************/
/**@name StatusFlags
 *
 * 	status flags for variables and section.
 * 	FIXME: not fully implemented.
 *	@see CFGSTATUS_ACTIVE
 *	@see CFGSTATUS_ALLOW_USER
 *	@see CFGSTATUS_ALLOW_OTHER_HOSTS
 *	@see CFGSTATUS_TEMPORARILY_DEACTIVATED
 */
/*********************************************************************/
typedef enum eCFGSTATUS
{
/*@{*/
	/** entry is active
	 */
	CFGSTATUS_ACTIVE			=  1,
	/** users may have their own config, overwriting
	 *  the default (host-) config
	 */
	CFGSTATUS_ALLOW_USER			=  2,
	/** config maybe read by other hosts, too
	 */
	CFGSTATUS_ALLOW_OTHER_HOSTS		=  4,
	/** entry ist temporarily deactivated
	 *  
	 *  useful for changing things in a configuration
	 *  temorarily. 
	 *  FIXME: possibly not needed (ACTIVE is suffucient?)
	 */
	CFGSTATUS_TEMPORARILY_DEACTIVATED	=  8
/*@}*/
} i_CFGSTATUS;

t_config* CfgCreate ();
void  CfgSetSeperator ( t_config* p_Cfg, char c_Seperator );
int   CfgGetError ( t_config* p_Cfg );
int   CfgReadConfig ( t_config** p_Cfg, char* psz_Name );
void  CfgFreeConfig ( t_config* p_Cfg );
void  CfgDump ( int fd, t_config* p_Cfg );
void  CfgSetStart ( t_config* p_Cfg );
void  CfgSetSecStart ( t_config* p_Cfg );
void  CfgSetVarStart ( t_config* p_Cfg );

int   CfgFindSection ( t_config* p_Cfg, char* psz_Name );
int   CfgFindVariable ( t_config* p_Cfg, char* psz_Name );
int   CfgFindSection_global ( t_config* p_Cfg, char* psz_Name );
int   CfgSetToVariable ( t_config* p_Cfg, char* psz_Varname );
int   CfgSetToSection ( t_config* p_Cfg, char* psz_Secname );

int   CfgAddSection ( t_config* p_Cfg, char* psz_Name,
	char* psz_Description, char* psz_Comment, i_CFGSTATUS n_Status );
int   CfgAddSubSection ( t_config* p_Cfg, char* psz_Name,
	char* psz_Description, char* psz_Comment, i_CFGSTATUS n_Status );
int   CfgDelSection ( t_config* p_Cfg, char* psz_Name );
char* CfgNextSection ( t_config* p_Cfg );
char* CfgNextSubSection ( t_config* p_Cfg );
char* CfgParentSection ( t_config* p_Cfg );
char* CfgNextVariable ( t_config* p_Cfg );
char* CfgPrevVariable ( t_config* p_Cfg );
int   CfgAddVariable ( t_config* p_Cfg, char* psz_Name,
	char* psz_Value, char* psz_Descrption, char* psz_Comment,
	i_CFGSTATUS n_Status );
/*
 * 
 * for current section
 *
 */
char* CfgGetSectionName ( t_config* p_Cfg );
char* CfgGetSectionDescription ( t_config* p_Cfg );
char* CfgGetSectionComment ( t_config* p_Cfg );
int   CfgGetSection_Status ( t_config* p_Cfg );
int   CfgGetSectionCount ( t_config* p_Cfg );

int   CfgSetSectionName ( t_config* p_Cfg, char* psz_Name );
int   CfgSetSectionDescription ( t_config* p_Cfg,
	char* psz_Description );
int   CfgSetSectionComment ( t_config* p_Cfg, char* psz_Comment );
int   CfgSetSectionStatus ( t_config* p_Cfg, i_CFGSTATUS n_Status );

/*
 * 
 * for variables
 *
 */

/* by name */
char* CfgGetVarValueOf ( t_config* p_Cfg, char* psz_Varname );
char* CfgGetVarDescriptionOf ( t_config* p_Cfg, char* psz_Varname );
char* CfgGetVarCommentOf ( t_config* p_Cfg, char* psz_Varname );
int   CfgGetVarStatusOf ( t_config* p_Cfg, char* psz_Varname );
char* CfgGetVarPredefOf ( t_config* p_Cfg, char* psz_Varname );

int   CfgSetVarValueOf ( t_config* p_Cfg, char* psz_Varname,
	char* psz_Value );
int   CfgSetVarDescriptionOf ( t_config* p_Cfg, char* psz_Varname,
	char* psz_Description );
int   CfgSetVarCommentOf ( t_config* p_Cfg, char* psz_Varname,
	char* psz_Comment );
int   CfgSetVarStatusOf ( t_config* p_Cfg, char* psz_Varname,
	i_CFGSTATUS n_Status );
int   CfgSetVarNameOf ( t_config* p_Cfg, char* psz_Varname,
	char* psz_NewName );
int   CfgSetVarPredefOf ( t_config* p_Cfg, char* psz_Varname,
	char* psz_Predef );


/* current */
int   CfgStore ( t_config* p_Cfg );
int   CfgRestore ( t_config* p_Cfg );
char* CfgGetVarNameOfNext ( t_config* p_Cfg );
char* CfgGetVarName ( t_config* p_Cfg );
char* CfgGetVarValue ( t_config* p_Cfg );
char* CfgGetVarDescription ( t_config* p_Cfg );
char* CfgGetVarComment ( t_config* p_Cfg );
int   CfgGetVarStatus ( t_config* p_Cfg );
char* CfgGetVarNextPredef ( t_config* p_Cfg );
int   CfgGetVarCount ( t_config* p_Cfg );

int   CfgSetVarValue ( t_config* p_Cfg, char* psz_Value );
int   CfgSetVarDescription ( t_config* p_Cfg, char* psz_Description );
int   CfgSetVarComment ( t_config* p_Cfg, char* psz_Comment );
int   CfgSetVarStatus ( t_config* p_Cfg, i_CFGSTATUS n_Status );
int   CfgSetVarName ( t_config* p_Cfg, char* psz_Name );
int   CfgSetVarPredef ( t_config* p_Cfg, char* p_Predef );

#endif
