/*
**	This file is part of cfgd
**
**      Copyright 2002 Oliver Schroeder
**
**      Permission to use, copy, modify, and distribute this
**      software and its documentation for any purpose and
**      without any fee is hereby granted, provided that the
**      above copyright notice and this permission notice
**      appear in all copies of the software and derivative
**      works or modified versions thereof, and that both the
**      copyright notice and this permission and disclaimer
**      notice appear in supporting documentation.
**
**      This software is provided "as is" WITHOUT ANY WARRANTY.
**
**      Oliver Schroeder <post@o-schroeder.de>
**
*/

#include <stdlib.h>
#include "cfgstack.h"

/*********************************************************************
 * create an empty new stack
 *********************************************************************/
t_CfgStack*
CfgStack_Create ()
{
	t_CfgStack* p_NewStack;

	p_NewStack = (t_CfgStack*) malloc (sizeof (t_CfgStack));
	if (p_NewStack == 0)
	{
		return (0);
	}
	p_NewStack->vp_Content	= 0;
	p_NewStack->p_Top	= 0;
	p_NewStack->p_Next	= 0;
	p_NewStack->p_Current	= 0;
	return (p_NewStack);
} /* CfgStack_Create() */

/*********************************************************************
 * push an element on the stack
 *********************************************************************/
int
CfgStack_Push
	(
	t_CfgStack*	p_Stack,
	void*		vp_Content
	)
{
	t_CfgStack* p_NewNode;
	
	p_NewNode = (t_CfgStack*) malloc (sizeof (t_CfgStack));
	if (p_NewNode == 0)
	{
		return (-1);
	}
	p_NewNode->vp_Content	= vp_Content;
	p_NewNode->p_Next	= p_Stack->p_Top;
	p_Stack->p_Top		= p_NewNode;
	return (0);
} /* CfgStack_Push() */

/*********************************************************************
 * return the first element from stack, leave the stack unchanged
 *********************************************************************/
void*
CfgStack_Top
	(
	t_CfgStack*	p_Stack
	)
{
	void *vp_Content;
	
	if ((p_Stack == 0) || (p_Stack->p_Top == 0))
	{
		return ((void*) 0);
	}
	vp_Content = p_Stack->p_Top->vp_Content;
	return (vp_Content);
} /* CfgStack_Pop() */

/*********************************************************************
 * pop the first element from stack
 *********************************************************************/
void*
CfgStack_Pop
	( 
	t_CfgStack*	p_Stack
	)
{
	t_CfgStack*	p_Node;
	void*		vp_Content;
	
	if ((p_Stack == 0) || (p_Stack->p_Top == 0))
	{
		return ((void*) 0);
	}
	p_Node		= p_Stack->p_Top;
	p_Stack->p_Top	= p_Node->p_Next;
	vp_Content	= p_Node->vp_Content;
	free (p_Node);
	p_Node		= 0;
	return (vp_Content);
} /* CfgStack_Pop() */

/*********************************************************************
 * return 1 if stack is empty, else 0
 *********************************************************************/
int
CfgStack_Empty
	(
	t_CfgStack*	p_Stack
	)
{
	if ((p_Stack == 0) || (p_Stack->p_Top == 0))
	{
		return (1);
	}
	return (0);
} /* CfgStack_Empty() */

/*********************************************************************
 * set current pointer to the first element
 *********************************************************************/
void
CfgStack_SetStart ( t_CfgStack* p_Stack )
{
	if ((p_Stack == 0) || (p_Stack->p_Top == 0))
	{
		return;
	}
	p_Stack->p_Current = p_Stack->p_Top;
}

/*********************************************************************
 * set current pointer to the next element
 *********************************************************************/
void*
CfgStack_Next ( t_CfgStack* p_Stack )
{
	void*	vp_Content;

	if ((p_Stack == 0) || (p_Stack->p_Current == 0))
	{
		return (0);
	}
	vp_Content = p_Stack->p_Current->vp_Content;
	p_Stack->p_Current = p_Stack->p_Current->p_Next;
	return (vp_Content);
}


/*********************************************************************
 * delete the stack
 *********************************************************************/
void
CfgStack_Destroy ( t_CfgStack* p_Stack )
{
	t_CfgStack* p_Node;
	t_CfgStack* p_Top;

	if (p_Stack == 0)
	{
		return;
	}
	p_Top = p_Stack->p_Top;
	while (p_Top != 0)
	{
		p_Node = p_Top->p_Next;
		free (p_Top);
		p_Top = p_Node;
	}
} /* CfgStack_Destroy() */

